#version 100

uniform mat4 diffuse_texture_mx;

#if defined(SPLIT_ALPHA)
uniform mat4 diffuse_alpha_texture_mx;
varying mediump vec2 frag_alpha_tex_coord;
#endif

attribute vec3 v_position;
attribute vec2 v_tex_coord_0;

#if defined(VARYING_COLOR)
attribute lowp vec4 v_color;
varying lowp vec4 frag_color;
#endif

#if defined(USE_SKINNING)

#if !defined(MAX_BONE_MATRICES)
#	define MAX_BONE_MATRICES 4
#endif

#if !defined(BONES_PER_VERTEX)
#	define BONES_PER_VERTEX 2
#endif

#if BONES_PER_VERTEX == 1
#	define BONE_VEC_TYPE float
#elif BONES_PER_VERTEX == 2
#	define BONE_VEC_TYPE vec2
#elif BONES_PER_VERTEX == 3
#	define BONE_VEC_TYPE vec3
#elif BONES_PER_VERTEX == 4
#	define BONE_VEC_TYPE vec4
#else
#	error BONES_PER_VERTEX must be 1, 2, 3, or 4
#endif

#if !defined(NO_BONE_WEIGHT)
attribute mediump BONE_VEC_TYPE v_bone_weights;
#endif

attribute mediump BONE_VEC_TYPE v_bone_indices;

// ES 2.0 does not support 4x3 matrix type, so we pass them as arrays of vec4's in row-major format
uniform vec4 bone_matrices[3 * MAX_BONE_MATRICES];

#endif

uniform mat4 model_view_projection_mx;

#if defined(FOG)
varying mediump float fog_factor;
uniform mat4 model_view_mx;
uniform int fog_falloff_mode;
uniform mediump float fog_density;
uniform vec2 fog_distances;
#endif

varying highp vec2 frag_tex_coord;

void main()
{
#if defined(USE_SKINNING)
	highp vec3 pos_sum = vec3(0.0, 0.0, 0.0);
	// highp vec4 pos = vec4(v_position, 1.0);

	mediump BONE_VEC_TYPE bone_weights = v_bone_weights;
	mediump BONE_VEC_TYPE bone_indices = v_bone_indices;

	for(int i = 0; i < BONES_PER_VERTEX; ++i)
	{
#if BONES_PER_VERTEX != 1
		int bone_index = int(bone_indices[i]) * 3;
#else
		int bone_index = int(bone_indices) * 3;
#endif

        vec4 b0 = bone_matrices[bone_index];
        vec4 b1 = bone_matrices[bone_index+1];
        vec4 b2 = bone_matrices[bone_index+2];

        float x = v_position.x;
        float y = v_position.y;
        float z = v_position.z;
        float w = 1.0;

#if !defined(NO_BONE_WEIGHT)

#   if BONES_PER_VERTEX != 1
        mediump float weight = bone_weights[i];
#   else
        mediump float weight = bone_weights;
#   endif

        // The following lines were the original implementation of this
        // bone application shader. However, on PowerVR-SGX5** GPUs, the
        // dot(2) function appears to be broken, producing wildly incorrect
        // vertex transforms. The accepted solution for this issue is to
        // roll our own dot product in the vert shader.

        // pos_sum.x += dot(bone_matrices[bone_index],   pos) * weight;
        // pos_sum.y += dot(bone_matrices[bone_index+1], pos) * weight;
        // pos_sum.z += dot(bone_matrices[bone_index+2], pos) * weight;

        pos_sum.x += (b0.x * x + b0.y * y + b0.z * z + b0.w * w) * weight;
        pos_sum.y += (b1.x * x + b1.y * y + b1.z * z + b1.w * w) * weight;
        pos_sum.z += (b2.x * x + b2.y * y + b2.z * z + b2.w * w) * weight;

#else
		pos_sum.x += (b0.x * x + b0.y * y + b0.z * z + b0.w * w);
        pos_sum.y += (b1.x * x + b1.y * y + b1.z * z + b1.w * w);
        pos_sum.z += (b2.x * x + b2.y * y + b2.z * z + b2.w * w);
#endif
	}

	gl_Position = model_view_projection_mx * vec4(pos_sum, 1.0);
#else
	gl_Position = model_view_projection_mx * vec4(v_position, 1.0);
#endif

	frag_tex_coord = (diffuse_texture_mx * vec4(v_tex_coord_0, 0, 1)).xy;

#if defined(SPLIT_ALPHA)
	frag_alpha_tex_coord = (diffuse_alpha_texture_mx * vec4(v_tex_coord_0, 0, 1)).xy;
#endif

#if defined(VARYING_COLOR)
	frag_color = v_color;
#endif

#if defined(FOG)
	// fog eye "distance"
	float d = abs((model_view_mx * vec4(v_position, 1.0)).z);

	if(fog_falloff_mode == 0) // linear
	{
		fog_factor = clamp((fog_distances.y - d) / (fog_distances.y - fog_distances.x), 0.0, 1.0);
	}
	else if(fog_falloff_mode == 1) // exp
	{
		fog_factor = clamp(exp(-fog_density * d), 0.0, 1.0);
	}
	else if(fog_falloff_mode == 2) // exp^2
	{
		fog_factor = clamp(exp(-fog_density * fog_density * d * d), 0.0, 1.0);
	}
#endif
}

